Theorie: Label, Buttons & Message-Boxen
=======================================

* * *

[TOC]

* * *


## Was ist Tkinter?

Erklären wir die Gleichung "GUI = tkinter + Python": **GUI** bedeutet **g**raphical **u**ser
**i**nterface, zu deutsch sind das die vielen Fenster, Buttons und Menüs, die
einem heute begegnen. Will man so etwas in Python schreiben, benötigt man eine
Fenster-Bibliothek, genau das leistet **tkinter**! Tkinter bedeutet **Tk
interface**, also eine Schnittstelle zu **Tk**, und _Tk_ ist eine "einfache"
Fensterbibliothek geschrieben in **Tcl** (eine Programmiersprache, Tcl steht
für _Tool command language_, Tk für _Tool kit_). Hier ein Beispiel (in
Python):

![](./media/tkinter_bsp01.jpg)

Die grafischen Elemente wie Buttons und Menüs haben einen Namen: **Widget**
(=**Wi**ndow Ga**dget**). Ein Widget hat normalerweise bestimmte
Eigenschaften, im Beispiel oben etwa die Hintergrundfarbe

![](./media/tkinter_bsp01_gelb.jpg)

und bestimmte Methoden, das sind Befehle, die die Widgets "verstehen". Ein
sehr wichtiger Befehl ist beispielsweise `pack`, mit diesem Befehl wird der
Text überhaupt erst angezeigt!



* * *

## Label-Widget

Das **Label-Widget** dient zur Beschriftung, das folgende Python-Programm
erzeugt ein Label:

    
    1    from tkinter import *
    2    root=Tk()
    3    lab=Label(root,text="Viel Spass mit dem Tkinter-Tutorial")
    4    lab.pack()
    5    root.mainloop()
  

Erklärung:

* In Zeile 1 wird die tkinter-Bibliothek importiert, sonst kennt Python nicht die tkinter-Befehle und beschwert sich. Statt Bibliothek sagt man in Python übrigens **Modul**.
* Zeile 2 sieht so ähnlich aus wie eine Zuweisung: Hier sagen wir Python, dass wir gerne eine Fenster-Anwendung mit Namen `root` hätten. Die Methode `Tk()` gibt uns dann freundlicherweise dieses Fenster zurück. Probiere in Python die folgenden beiden Zeilen aus:<br />`>>> from tkinter import *`  <br />`>>> root=Tk()`<br />


* In Zeile 3 erzeugen wir unser Label, wir sagen Python, dass das Label ins Fenster `root` hinein gehört, und welcher Text als Label (=Beschriftung) ausgegeben werden soll.
* Die Methode `pack` sorgt dafür, dass unser Text auch angezeigt wird. Überzeuge dich davon, indem du Zeile 3 & 4 in Python eingibst.
* Der Befehl `mainloop` in `root.mainloop()` (Zeile 5) steht für eine Endlosschleife (!), unser Tkinter-Skript wartet dann auf so interessante Ereignisse wie einen Maus-Klick oder eine Maus-Bewegung. 



* * *

## Button-Widget

Buttons sind zum Klicken da, in Tkinter geht das so:
    
    
    1    from tkinter import *
    2    root=Tk()
    3    but=Button(root,text="Wo ist Tommy?")
    4    but.pack()
    5    root.mainloop()
    

Wirkung:

![](./media/tkinter_bsp02_tommy.jpg)

Erklärung:

* Zeile 1, 2 und 4 kennen wir schon (?), wobei Zeile 4 so ähnlich wirkt wie beim Label, nur wird diesmal eben der Button angezeigt.
* Bleibt also noch Zeile 3: Ein Button im root-Fenster, beschriftet mit "Wo ist Tommy?". Die große Ernüchterung kommt beim Klicken: es passiert . . . _nichts_! Was tun?

Überlegen wir: jedes Mal, wenn auf den Button geklickt wird, soll etwas
passieren, sonst natürlich nicht. Wir müssen also den Klick mit etwas
verbinden, beispielsweise mit einer Ausgabe, für die Ausgabe haben wir Labels,
also verbinden wir unseren Button mit der Ausgabe eines Labels. Dafür gibt es
in Tkinter den `command`-Parameter:
   
    
    1    from tkinter import *
    2    def antwort():
    3        lab=Label(root,text="Hier nicht!")
    4        lab.pack()
    5
    6    root=Tk()
    7    but=Button(root,text="Wo ist Tommy?", command=antwort)
    8    but.pack()
    9    root.mainloop()
    

Erklärung:

* In Zeile 2-4 wird die Funktion `antwort` definiert, aber wo wird sie aufgerufen?  `____________________________________________`

* Die einzige Änderung ist in Zeile 7: In der Button-Vereinbarung wurde ein Parameter hinzugefügt: `command`, das macht man so ähnlich auch in HTML (klar?)

Jetzt drücken wir mehrmals auf den Button, aber überzeugend, na ja, für den
Anfang . . .



* * *

## Message-Boxen

Was wir brauchen, sind natürlich "Messsage-Boxen":

![](./media/tkinter_bsp03_tommy.jpg)

Und wie geht das in Python? Einfach ein Zweizeiler:

    
    
    1    from tkinter import messagebox
    2    messagebox.showinfo('Hier nicht!','Hier auch nicht!')
    

Erklärung:

* In Zeile 1 wird wieder eine Bibliothek importiert, diesmal aber `messageBox`, dieses Modul enthält die ganzen Messsage-Boxen.
* Und in Zeile 2 rufen wir eine der Dialog-Boxen auf: `showinfo`. Frage: Warum müssen wir showinfo zwei Parameter übergeben?  `______________________________________________________________________`<br /><br />`______________________________________________________________________`

  

Welche Message-Boxen ausser `showinfo` gibt es noch? Hier eine kleine
Übersicht:


   * `showwarning()`
   * `showerror()`
   * `askyesno()`
   * `askokcancel()`
   * `askretrycancel()`


Der Aufruf geht fast genauso wie bei `showinfo`, einfach ausprobieren, wir
aber kommen zum Programm des Tages:

    
    1    from tkinter import *
    2    from tkinter import messagebox
    3    def antwort():
    4        messagebox.showinfo('Hier nicht!','Hier auch nicht!')
    5
    6    root=Tk()
    7    but=Button(root,text="Wo ist Tommy?",command=antwort)
    8    but.pack()
    9    root.mainloop()
    

* * *

##  Aufgaben
  
1. Labels kann man bunt machen, probiere (in IDLE) `background="blue"` oder `foreground="magenta"`<br /><br />
2. Es muss nicht immer Text sein . . ., also ersetzen wir im Button den `text="Wo ist Tommy?"` durch Bilder: `bitmap="questhead"`. Statt `questhead` darfst du auch schreiben: `error, gray75, gray50, gray25, gray12, hourglass, info, question` oder `warning`. (`gray` sieht etwas komisch aus . . .) <br /><br />
3. Teste Message-Boxen!<br /><br />
4. Wenn man in IDLE mit Message-Boxen experimentiert, gibt Python Meldungen zurück, was sollen diese Meldungen?


 
* * *


<hr style="page-break-after:always">

&rarr; sp, 2023-07-28

* * *
* * *
